/* Klaus Thiele (kth@oblib.teuto.de) - June '95 */

#ifndef ERR_SQL
#define ERR_SQL

#define FETEEOF   1      /* End of Fetch                                  */

#define SQLESTS   102    /* Select buffer is too small                    */
#define SQLEDNN   104    /* Attempt to fetch non-numeric data into numeric column  */
#define SQLENOF   105    /* Attempt to fetch into numeric column that is too small */
#define SQLEDTN   106    /* Data type not supported                       */
#define SQLETMB   113    /* Too many binds                                */
#define SQLEIVC   114    /* Invalid bind name or bind number              */
#define SQLEMSC   118    /* Must follow a successful compile of a SELECT statement  */
#define SQLEICN   135    /* Invalid cursor number                         */
#define SQLECCS   136    /* Out of memory at the client workstation       */
#define SQLEIRN   144    /* Invalid row number                            */
#define SQLE058   158    /* msqlQuery error         (NOT USED by SQLBase) */
#define SQLERNA   163    /* Result sets not active                        */
#define SQLE065   165    /* No Field list           (NOT USED by SQLBase) */
#define SQLE071   171    /* Field not found         (NOT USED by SQLBase) */
#define SQLE073   173    /* Bind data ptr is NULL   (NOT USED by SQLBase) */
#define SQLE074   174    /*                         (NOT USED by SQLBase) */
#define SQLECNC   187    /* Cannot connect to server                      */

#define DBOENEX   203    /* Command has not been executed                 */
#define DBAECOD   401    /* Cannot locate the specified database          */

#define PRSEMFC   902    /* Missing FROM clause                           */
#define PRSEMIK   911    /* Missing INTO keyword                          */
#define PRSEISC   917    /* Invalid SQL statement                         */

#define TLKETND  5123    /* Table not found in dictionary                 */

#endif
