CREATE TABLE SAVINGS (SAV_ACC_NO INTEGER, SAV_DOLLARS REAL)
\g
CREATE TABLE CHECKING (CHK_ACC_NO INTEGER, CHK_DOLLARS REAL)
\g
INSERT INTO SAVINGS (SAV_ACC_NO,SAV_DOLLARS) VALUES (591,1000)
\g
INSERT INTO CHECKING (CHK_ACC_NO,CHK_DOLLARS) VALUES (1497,1000)
\g
