CREATE TABLE EMP (EMP_NAME CHAR(20), EMP_NO INTEGER, EMP_DOB CHAR(10))
\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('UMANG G',1001,'11-01-84')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('BRUCE S',2001,'11-01-84')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('DAVID I',2002,'12-21-84')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('DAVID R',2003,'08-01-85')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('DENNIS N',2004,'09-01-85')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('MIKE G',2005,'04-01-86')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('MIKE K',2006,'04-01-86')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('JEAN J',1002,'07-01-86')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('TRUDY R',1003,'10-01-86')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('STEVE B',2007,'12-01-86')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('EWA K',1004,'02-01-87')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('ED W',3001,'04-01-87')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('TOM H',1004,'06-01-87')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('MARK S',2008,'07-01-87')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('RICH H',1005,'10-01-87')\g
INSERT INTO EMP (EMP_NAME,EMP_NO,EMP_DOB) VALUES ('LINDA H',1006,'10-01-87')\g
#
CREATE TABLE TMP (TMP_NO INTEGER, TMP_NAME CHAR(20))
/g
