CREATE TABLE EMP (EMP_NO INTEGER, EMP_NAME CHAR(20))
\g
CREATE TABLE BONUS (BONUS_EMP_NO INTEGER, BONUS_AMOUNT REAL)
\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1001,'Mike Heath')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1002,'Larry Herndon')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1003,'Pat Sheridan')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1004,'Billy Bean')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1005,'Matt Nokes')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1006,'Dave Bergman')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1007,'Allan Trammell')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1008,'Daryl Evans')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1009,'Tom Brookens')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1010,'Kirk Gibson')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1011,'Lou Whitaker')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1012,'Chet Lemon')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1013,'Daryl Coles')\g
INSERT INTO EMP (EMP_NO,EMP_NAME) VALUES (1014,'John Grubb')\g
#
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1001,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1002,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1003,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1004,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1005,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1006,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1007,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1008,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1009,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1010,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1011,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1012,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1013,0)\g
INSERT INTO BONUS (BONUS_EMP_NO,BONUS_AMOUNT) VALUES (1014,0)\g
#
